gdjs.LoadingScreenPixiRenderer = function(runtimeGamePixiRenderer, loadingScreenSetup) {
  this._pixiRenderer = runtimeGamePixiRenderer.getPIXIRenderer();
  if (!this._pixiRenderer) {
    // A PIXI Renderer can be missing during tests, when creating a runtime game
    // without a canvas.
    return;
  }

  this._loadingScreen = new PIXI.Container();
  var bg = new PIXI.Sprite(PIXI.Texture.WHITE);
  // this._loadingScreen.width = window.screen.width*2;
  // this._loadingScreen.height = window.screen.height*2;
  bg.width = window.screen.width*window.devicePixelRatio;
  bg.height = window.screen.height*window.devicePixelRatio;
  bg.tint = 0xffffff;
  this._loadingScreen.addChild(bg);


  this._progressText = new PIXI.Text(' ', {
    fontSize: '30px',
    fontFamily: 'Arial',
    fill: '#000000',
    align: 'center',
  });
  this._loadingScreen.addChild(this._progressText);

  if (loadingScreenSetup && loadingScreenSetup.showGDevelopSplash) {
    this._madeWithText = new PIXI.Text('Made with', {
      fontSize: '30px',
      fontFamily: 'Arial',
      fill: '#000000',
      align: 'center',
    });
    this._madeWithText.position.y = this._pixiRenderer.height / 2 - 130;
    this._websiteText = new PIXI.Text('lieyou888.com', {
      fontSize: '30px',
      fontFamily: 'Arial',
      fill: '#000000',
      align: 'center',
    });
    this._websiteText.position.y = this._pixiRenderer.height / 2 + 100;

    this._splashImage = new PIXI.Sprite.fromImage(gdjs.splashImage);
    // this._splashImage.position.x = this._pixiRenderer.width / 2;
    // this._splashImage.position.y = this._pixiRenderer.height / 2;
    this._splashImage.width = 358;
    this._splashImage.height = 120;
    this._splashImage.position.x = bg.width / 2 - this._splashImage.width/2;
    this._splashImage.position.y = bg.height / 2 - this._splashImage.height/2;
    this._splashImage.anchor.x = 0.5;
    this._splashImage.anchor.y = 0.5;
    
    this._loadingScreen.addChild(this._splashImage);
    // this._loadingScreen.addChild(this._madeWithText);
    // this._loadingScreen.addChild(this._websiteText);

    console.debug(
      `_pixiRenderer.width: ` + this._pixiRenderer.width 
      + ';  position.x: ' + this._splashImage.position.x + ';  position.y: ' + this._splashImage.position.y
      + ';  anchor.x: ' + this._splashImage.anchor.x + ';  anchor.y: ' + this._splashImage.anchor.y 
      + ';  scale.x: ' + this._splashImage.scale.x + ';  scale.y: ' + this._splashImage.scale.y
      + ';  splashImage.width: ' + this._splashImage.width + ';  splashImage.height: ' + this._splashImage.height
      + ';  window.screen.width: ' + window.screen.width + ';  window.screen.height: ' + window.screen.height
      + ';  window.screen.availWidth: ' + window.screen.availWidth + ';  window.screen.availHeight: ' + window.screen.availHeight  
      + ';  window.devicePixelRatio: ' + window.devicePixelRatio
    );
  }
};

gdjs.LoadingScreenRenderer = gdjs.LoadingScreenPixiRenderer; //Register the class to let the engine use it.

gdjs.LoadingScreenPixiRenderer.prototype.render = function(percent) {
  if (!this._pixiRenderer) {
    return;
  }

  var screenBorder = 10;

  // if (this._madeWithText) {
  //   this._madeWithText.position.x =
  //     this._pixiRenderer.width / 2 - this._madeWithText.width / 2;
  //   this._madeWithText.position.y =
  //     this._pixiRenderer.height / 2 -
  //     this._splashImage.height / 2 -
  //     this._madeWithText.height -
  //     20;
  // }
  // if (this._websiteText) {
  //   this._websiteText.position.x =
  //     this._pixiRenderer.width - this._websiteText.width - screenBorder;
  //   this._websiteText.position.y =
  //     this._pixiRenderer.height - this._websiteText.height - screenBorder;
  // }


  this._progressText.text = percent + '%';
  this._progressText.position.x = this._pixiRenderer.width - screenBorder - 100;
  this._progressText.position.y = this._pixiRenderer.height - screenBorder - this._progressText.height ;

  this._pixiRenderer.render(this._loadingScreen);
};

gdjs.LoadingScreenPixiRenderer.prototype.unload = function() {
  // Nothing to do
};
